SystemOrganization addCategory: #'Slime-Core'!

!ParseTreeEnvironment methodsFor: '*slime' stamp: 'lr 12/17/2007 10:48'!
matcher
	^ matcher ifNil: [ matcher := ParseTreeSearcher new ]! !

Object subclass: #Slime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Slime-Core'!

Slime class
	instanceVariableNames: 'allCallAnswerSelectors allBrushSelectors allCallbackSelectors superSelectors deprecatedSelectors'!

Slime class
	instanceVariableNames: 'allCallAnswerSelectors allBrushSelectors allCallbackSelectors superSelectors deprecatedSelectors'!

!Slime class methodsFor: 'accessing' stamp: 'lr 12/17/2007 11:19'!
allBrushSelectors
	^ allBrushSelectors! !

!Slime class methodsFor: 'accessing' stamp: 'lr 12/17/2007 11:29'!
allCallAnswerSelectors
	^ allCallAnswerSelectors! !

!Slime class methodsFor: 'accessing' stamp: 'lr 12/17/2007 11:09'!
allCallbackSelectors
	^ allCallbackSelectors! !

!Slime class methodsFor: 'accessing' stamp: 'lr 12/17/2007 23:04'!
deprecatedSelectors
	^ deprecatedSelectors! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/17/2007 23:03'!
initialize
	self initializeAllBrushSelectors.
	self initializeAllCallbackSelectors.
	self initializeAllCallAnswerSelectors.
	self initializeDeprecatedSelectors.
	self initializeSuperSelectors! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/17/2007 11:19'!
initializeAllBrushSelectors
	| matcher |
	matcher := ParseTreeSearcher new.
	matcher 
		matchesAnyMethodOf: #( 
			'`selector ^ self tag: `#tag'
			'`selector ^ self brush: `@obj'
			'`selector: `aBlock self `selector with: `aBlock'
			'`selector ^ `class on: self'
		) do: [ :context :node | true ].
	allBrushSelectors := Set new.
	WACanvas allSubclassesDo: [ :class | 
		class methodDictionary keysAndValuesDo: [ :selector :method |
			(matcher executeTree: method parseTree initialAnswer: false) 
				ifTrue: [ allBrushSelectors add: selector ] ] ]! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/18/2007 17:50'!
initializeAllCallAnswerSelectors
	allCallAnswerSelectors := #( answer answer: call: show: show:onAnswer: show:onAnswer:delegation: lightbox: chooseFrom: chooseFrom:caption: confirm: inform: request: request:default: request:label: request:label:default: ) asSet! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/17/2007 10:38'!
initializeAllCallbackSelectors
	allCallbackSelectors := #( callback: callback:value: defaultAction: triggerArgument:callback: triggerAutocompleter: triggerInPlaceEditor: triggerPassenger: triggerSliderCallback: triggerSortable:callback: triggerTree:callback: ) asSet! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/17/2007 23:04'!
initializeDeprecatedSelectors
	| selectors |
	selectors := #( deprecatedApi deprecatedApi: ).
	deprecatedSelectors := Set new.
	Object allSubclassesDo: [ :class |
		class methodDictionary keysAndValuesDo: [ :selector :method |
			method literalsDo: [ :each |
				(selectors includes: each)
					ifTrue: [ deprecatedSelectors add: selector ] ] ] ].
	#( new text: renderDeprecatedOn: )
		do: [ :each | deprecatedSelectors remove: each ifAbsent: [ ] ]! !

!Slime class methodsFor: 'initialization' stamp: 'lr 12/17/2007 22:48'!
initializeSuperSelectors
	superSelectors := Dictionary new.
	superSelectors
		at: WAComponent name put: #( initialize updateRoot: updateStates: updateUrl: initialRequest: ) asSet;
		at: WATagBrush name put: #( initialize setParent:canvas: #with: )! !

!Slime class methodsFor: 'testing' stamp: 'lr 12/15/2007 22:23'!
isBrushSelector: aString
	Symbol 
		hasInterned: aString 
		ifTrue: [ :symbol | ^ allBrushSelectors includes: symbol ].
	^ false! !

!Slime class methodsFor: 'testing' stamp: 'lr 12/17/2007 10:14'!
isCallbackSelector: aString
	Symbol 
		hasInterned: aString 
		ifTrue: [ :symbol | ^ allCallbackSelectors includes: symbol ].
	^ false! !

!Slime class methodsFor: 'accessing' stamp: 'lr 12/17/2007 15:57'!
superSelectors
	^ superSelectors! !

!BlockLintRule class methodsFor: '*slime-bugs' stamp: 'lr 1/25/2008 20:49'!
callAnswerWhileRendering
	| matcher matches |
	matcher := ParseTreeSearcher new.
	matcher 
		matchesAnyOf: (Slime allCallAnswerSelectors collect: [ :each | 
			'`receiver' , (ParseTreeLintRule genericPatternForSelector: each) ])			
		do: [ :node :answer |
			(node parents noneSatisfy: [ :each |
				each isBlock
					and: [ each parent isMessage
					and: [ Slime isCallbackSelector: each parent selector ] ] ])
				ifTrue: [ answer add: node ].
			answer ].
	^ self new
		name: '#call:/#answer: while rendering';
		rationale: '#call: and #answer: should only be used from callback code, not within the rendering code.';
		methodBlock: [ :context :result |
			context isRenderingMethod ifTrue: [
				matches := matcher executeTree: context parseTree initialAnswer: OrderedCollection new.
				matches do: [ :each |
					result addClass: context selectedClass selector: context selector.
					result addSearchString: each selector ] ] ]! !

!BlockLintRule class methodsFor: '*slime-possible bugs' stamp: 'lr 12/17/2007 13:25'!
changesStateWhileRendering
	| matcher matches |
	matcher := ParseTreeSearcher new.
	matcher 
		matches: '`var'
		do: [ :node :answer |
			(node isWrite and: [
				node parents noneSatisfy: [ :each |
					each isBlock
						and: [ each parent isMessage
						and: [ Slime isCallbackSelector: each parent selector ] ] ] ])
				ifTrue: [ answer add: node ].
			answer ].
	^ self new
		name: 'Changes state while rendering';
		rationale: 'Application state should not be changed in the rendering code, use a callback to define state.';
		resultClass: ParseTreeEnvironment;
		methodBlock: [ :context :result |
			| vars |
			context isRenderingMethod ifTrue: [
				matches := matcher executeTree: context parseTree initialAnswer: OrderedCollection new.
				matches isEmpty ifFalse: [
					vars := context instVarNames.
					matches do: [ :each |
						(vars includes: each name) ifTrue: [
							result addClass: context selectedClass selector: context selector.
							result matcher
								matches: each name , ' := ``@obj'
								do: [ :node :answer | 
									answer isNil 
										ifTrue: [ node ] 
										ifFalse: [ answer ] ] ] ] ] ] ]! !

!BlockLintRule class methodsFor: '*slime-bugs' stamp: 'lr 1/25/2008 20:30'!
doesNotSendSuperInitialize
	^ self new
		name: 'Does not send super';
		rationale: 'Always send super when overriding specific hook methods.';
		methodBlock: [ :context :result |
			| class selectors |
			class := context selectedClass allSuperclasses
				detect: [ :each | Slime superSelectors includesKey: each name ]
				ifNone: [ Object ].
			selectors := Slime superSelectors at: class name ifAbsent: [ #() ].
			((selectors includes: context selector)
				and: [ (context parseTree superMessages includes: context selector) not ])
				ifTrue: [ result addClass: context selectedClass selector: context selector ] ]! !

!BlockLintRule class methodsFor: '*slime-bugs' stamp: 'lr 1/25/2008 20:55'!
instantiatesComponentWhileRendering
	| matcher matches |
	matcher := ParseTreeSearcher new.
	matcher 
		matches: '`receiver `@message: `@args'			
		do: [ :node :answer |
			| class | 
			(node receiver isVariable
				and: [ (class := Smalltalk classNamed: node receiver name) notNil 
				and: [ class includesBehavior: WAComponent ] ]) ifTrue: [
					(node parents noneSatisfy: [ :each |
						each isBlock
							and: [ each parent isMessage
							and: [ Slime isCallbackSelector: each parent selector ] ] ])
						ifTrue: [ answer add: node ] ].
			answer ].
	^ self new
		name: 'Instantiates component while rendering';
		rationale: 'Components should only be instanciated in initialization-code, callbacks or through lazy initialization.';
		methodBlock: [ :context :result |
			context isRenderingMethod ifTrue: [
				matches := matcher executeTree: context parseTree initialAnswer: OrderedCollection new.
				matches do: [ :each |
					result addClass: context selectedClass selector: context selector.
					result addSearchString: each receiver name ] ] ] ! !

!BlockLintRule class methodsFor: '*slime-possible bugs' stamp: 'lr 1/25/2008 20:50'!
sendsDeprecatedMessage
	^ self new
		name: 'Sends deprecated message';
		rationale: 'Deprecated selectors will be removed with the next release of Seaside.';
		methodBlock: [ :context :result |
			context messages do: [ :each | 
				(Slime deprecatedSelectors includes: each) ifTrue: [ 
					result addClass: context selectedClass selector: context selector.
					result addSearchString: each ] ] ]! !

!ParseTreeLintRule class methodsFor: '*slime-miscellaneous' stamp: 'lr 12/17/2007 22:53'!
avoidUnnecessaryWith
	| matcher |
	matcher := ParseTreeSearcher new.
	matcher
		matches: '`html `msg with: ``@arg'
		do: [ :node :answer |
			(answer isNil
				and: [ node parent isCascade not
				and: [ (Slime isBrushSelector: node receiver selector)
				and: [ (Slime isBrushSelector: node receiver selector , ':')
				and: [ (Slime isBrushSelector: node methodNode selector) not ] ] ] ])
				ifTrue: [ node ] ].
	^ self new
		name: 'Avoid unnecessary #with:';
		rationale: 'Sending #with: is only required if attributes are specified too.';
		matcher: matcher;
		yourself! !

!ParseTreeLintRule class methodsFor: '*slime-miscellaneous' stamp: 'lr 12/18/2007 17:54'!
extractCallbackCodeToMethod
	| matcher |
	matcher := ParseTreeSearcher new.
	matcher 
		matchesAnyOf: (Slime allCallbackSelectors collect: [ :each |
			'`html `msg' , (self genericPatternForSelector: each) ])
		do: [ :node :answer |
			(answer isNil and: [ Slime isBrushSelector: node receiver selector ]) ifTrue: [
				node arguments 
					detect: [ :each | each isBlock and: [ each body statements size > 1 ] ]
					ifNone: [ nil ] ] ].
	^ self new
		name: 'Extract callback code to separate method';
		rationale: 'For clarity rendering code and callback code should not be mixed, extract the contents of the callback block to a separate method.';
		matcher: matcher;
		yourself! !

!ParseTreeLintRule class methodsFor: '*slime-miscellaneous' stamp: 'lr 12/17/2007 22:51'!
unnecessaryBlockPassedToBrush
	| matcher |
	matcher := ParseTreeSearcher new.
	matcher 
		matchesAnyOf: #( 
			'`html `msg: [ ]' 
			'`html `msg: [ `html text: ``@arg ]' 
			'`html `msg: [ `html render: ``@arg ]' ) 
		do: [ :node :answer |
			(answer isNil and: [ Slime isBrushSelector: node selector ])
				ifTrue: [ node arguments first ] ].
	matcher
		matchesAnyOf: #(
			'`html `msg with: [ ]' 
			'`html `msg with: [ `html text: ``@arg ]' 
			'`html `msg with: [ `html render: ``@arg ]' )
		do: [ :node :answer |
			(answer isNil and: [ Slime isBrushSelector: node receiver selector ])
				ifTrue: [ node arguments first ] ].
	^ self new
		name: 'Unnecessary block passed to brush';
		rationale: 'Sending a block as argument to #with: is only needed when nesting brushes.';	
		matcher: matcher;
		yourself! !

!ParseTreeLintRule class methodsFor: '*slime-bugs' stamp: 'lr 12/17/2007 13:38'!
withHasToBeLastMessageInCascade
	| matcher |
	matcher := ParseTreeSearcher new.
	matcher 
		matches: '`html `msg with: ``@arg'  
		do: [ :node :answer |
			(answer isNil
				and: [ node parent isCascade
				and: [ (node parent messages last = node) not
				and: [ (Slime isBrushSelector: node receiver selector) ] ] ])
				ifTrue: [ node ] ].
	^ self new
		name: '#with: has to be last message in cascade';
		rationale: 'Sending #with: triggers serialization of the brush attributes, any attribute being specified afterwards has no effect.';
		matcher: matcher;
		yourself! !

!RBProgramNode methodsFor: '*slime' stamp: 'lr 12/17/2007 10:02'!
parents
	^ parent isNil
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [
			parent parents
				addLast: self;
				yourself ]! !

!SmalllintContext methodsFor: '*slime' stamp: 'lr 12/17/2007 09:55'!
isRenderingMethod
	(self selector numArgs > 0 and: [ self selector beginsWith: #render ])
		ifFalse: [ ^ false ].
	self compiledMethod literalsDo: [ :each |
		(each isSymbol and: [ Slime isBrushSelector: each ])
			ifTrue: [ ^ true ] ].
	^ false! !
Slime initialize!
